/* General */
:root {
    --container-width: 800px;
    --color-primary-lightest: #ffeec5;
    --color-primary: #ff991f;
    --color-primary-darker: #dc7f0f;
    --color-text: #636363;
    --color-grey: #545454;
    --color-grey-light: #f9f9f9;
    --font-size: 16px;

    --jp-content-heading-line-height: 1.5em;

    --jp-content-font-family: "Open Sans", Arial, Helvetica, sans-serif;
    --jp-content-heading-font-weight: bold;
    --jp-content-link-color: var(--color-primary);

    --jp-content-font-size2: 1.5rem;
    --jp-content-font-size3: 1.75rem;
    --jp-content-font-size4: 2rem;
    --jp-content-font-size5: 2.25rem;

    --dark-color-primary-lightest: #26273D;
    /* --dark-color-primary: #7A8AFF; */
    --dark-color-primary: #f77e14;
    --dark-color-primary-darker: #303886;
    --dark-color-text: hsla(0,0%,100%,.7);
    --dark-color-grey: #ababab;
    --dark-color-grey-light: #202020;
}

html,
body,
p,
ul,
li {
    color: var(--color-text);
    font-size: var(--font-size);
}
.theme--dark p, .theme--dark ul, .theme--dark li {
    color: var(--dark-color-text);
}

a {
    transition: color 0.3s ease;
}

a:link {
    color: var(--color-primary);
}

.theme--dark a:link {
    color: var(--dark-color-primary);
}

.theme--dark .jp-RenderedHTMLCommon a:visited {
    color: var(--dark-color-primary);
}

.jp-RenderedHTMLCommon pre {
    background-color: var(--color-grey-light);
    display: block;
    border-radius: 1rem;
    padding: 1.75rem 2.5rem;
    margin: 0.5rem 0;
}

.theme--dark .jp-RenderedHTMLCommon pre {
    background-color: var(--dark-color-grey-light);
}

.code {
    padding: 0;
    margin: 0;
}

.text-center {
    text-align: center;
}

.bg-primary {
    background-color: var(--color-primary);
}

.theme--dark .bg-primary {
    background-color: var(--dark-color-primary);
}

.bg-primary-fade {
    background-color: var(--color-primary-lightest) !important;
}

.theme--dark .bg-primary-fade {
    background-color: var(--dark-color-primary-lightest) !important;
}

.padding-40 {
    padding: 40px !important;
}

h1 {
    font-size: 2.4rem;
}

.bg-grey {
    background-color: var(--color-grey-light);
}

.theme--dark .bg-grey {
    background-color: var(--dark-color-grey-light);
}

li::marker {
    color: var(--color-primary);
    font-size: 1.2em;
    font-weight: bold;
}

.theme--dark li::marker {
    color: var(--dark-color-primary);
}

blockquote,
.jp-RenderedHTMLCommon blockquote {
    font-family: "Courier New", Courier, monospace;
    border-left: 8px solid var(--color-primary);
    margin: 1rem 0;
    padding: 1rem 1rem 1rem 4rem;
    position: relative;
    z-index: 0;
    background: no-repeat left center url("data:image/svg+xml,%3Csvg width='25' height='25' fill='none' xmlns='http://www.w3.org/2000/svg'%3E%3Cg clip-path='url(%23a)' fill='%23FF991F'%3E%3Cpath d='M12.24 0C5.49 0 0 5.49 0 12.24s5.49 12.24 12.24 12.24 12.24-5.49 12.24-12.24S19 0 12.24 0zm0 22.26c-5.52 0-10.02-4.49-10.02-10.02 0-5.53 4.5-10.01 10.02-10.01s10.02 4.49 10.02 10.02c0 5.53-4.49 10.02-10.02 10.02v-.01z'/%3E%3Cpath d='M12.24 9.65c-.61 0-1.11.5-1.11 1.11v7.42c0 .61.5 1.11 1.11 1.11.61 0 1.11-.5 1.11-1.11v-7.42c0-.61-.5-1.11-1.11-1.11zM12.24 5.19a1.13 1.13 0 0 0-1.12 1.12c0 .3.12.58.33.79a1.112 1.112 0 0 0 1.58 0 1.112 1.112 0 0 0 0-1.58c-.21-.21-.49-.33-.79-.33z'/%3E%3C/g%3E%3Cdefs%3E%3CclipPath id='a'%3E%3Cpath fill='%23fff' d='M0 0h24.49v24.49H0z'/%3E%3C/clipPath%3E%3C/defs%3E%3C/svg%3E");
    background-color: var(--color-grey-light);
    background-position: 1.2rem center;
}

.theme--dark blockquote,
.theme--dark .jp-RenderedHTMLCommon blockquote {
    border-left: 8px solid var(--dark-color-primary);
    background-color: var(--dark-color-grey-light);
}

blockquote p:last-child {
    margin-bottom: 0;
}

.footer-wrapper{
    width: 100%;
    padding: 50px 0;
    flex-wrap: wrap;
    row-gap: 2.5em;
}

.row-container{
    width: 60%;
    flex-wrap: wrap;
}
.row-container > div{
    max-width: 500px;
}

.v-btn.homepage-button {
    background-color: var(--color-primary-darker) !important;
}

.theme--dark .v-btn.homepage-button {
    background-color: var(--dark-color-primary) !important;
    color: var(--dark-color-grey-light);
}

.v-card.testimonial-card{
    background-color: var(--color-primary-lightest);
    display: flex;
    flex-direction: column;
    justify-content: space-between;
}

.theme--dark .v-card.testimonial-card{
    background-color: var(--dark-color-primary-lightest);
}

/* vuetify component overrides */

.v-btn.solara-docs-button {
    font-weight: bold;
    border-radius: 10px;
}

.btn-size--xlarge {
    height: 4.25rem !important;
    padding: 0 2rem !important;
    font-size: 1.125rem !important;
}

/* overriding styles that didn't set primary color */
.theme--light.v-application .text--primary {
    color: var(--color-primary-darker) !important;
}

.theme--dark.v-application .text--primary {
    color: var(--dark-color-primary-darker) !important;
}

/* header */
.news {
    background-color: var(--color-primary);
    padding: 10px;
    font-size: 20px;
    display: flex;
    justify-content: center;
}
.theme--dark .news {
    background-color: var(--dark-color-primary);
}
.news a {
    color: white !important;
}

/* hero */
.hero {
    background: var(--color-primary-lightest) url(https://dxhl76zpt6fap.cloudfront.net/public/hero.webp);
    background-size: cover;
    min-height: 660px;
    padding: 7rem 0;
}

.theme--dark .hero {
    background: var(--dark-color-primary-lightest) url(https://dxhl76zpt6fap.cloudfront.net/public/hero.webp);
    background-size: cover;
}

.hero h1 {
    font-size: 4.5rem;
}

.hero h2 {
    padding-left: 150px;
    padding-right: 150px;
}

.hero b {
    color: var(--color-primary);
}

.theme--dark .hero b {
    color: var(--dark-color-primary);
}

@media screen and (max-width: 960px) {
    .hero {
        padding: 4rem 0;
        min-height: auto;
    }

    .hero h1 {
        font-size: 3rem;
    }
}

/* fixes padding issue on drawer menu */
.v-list-group__items {
    padding-left: 1rem;
}

/* menu */

.menu {
    list-style: none;
    display: flex;
}

.menu li a:link,
.menu li a:visited {
    color: var(--color-grey);
}

.theme--dark .menu li a:link,
.theme--dark .menu li a:visited {
    color: var(--dark-color-grey);
}

.menu li a:hover {
    color: var(--color-primary);
}

.theme--dark .menu li a:hover {
    color: var(--dark-color-primary);
}

.menu li a:link {
    display: block;
    padding: 1rem 2rem;
}

.menu .active {
    font-weight: bolder;
}

/* sidebar */

.sidebar {
    display: flex;
    justify-content: right;
    padding: 4rem;
}

.sidebar ul li {
    font-size: 1.3rem;
}

.sidebar ul a {
    display: block;
}

.sidebar a:link,
.sidebar a:visited {
    color: var(--color-grey);
}

.theme--dark .sidebar a:link,
.theme--dark .sidebar a:visited {
    color: var(--dark-color-grey);
}

.sidebar a:hover {
    color: var(--color-primary);
    text-decoration: none;
}

.theme--dark .sidebar a:hover {
    color: var(--dark-color-primary);
}

.sidebar ul li {
    font-weight: normal;
    margin: 0.8rem auto;
}

.sidebar>div>ul>li {
    font-weight: bold;
    margin: 1.4rem auto;
}

.sidebar li ul {
    margin: 0.8rem 0;
}

.sidebar>div>ul>li>a:link,
.sidebar>div>ul>li>a:visited {
    color: black;
}

.sidebar li.active {
    font-weight: bolder;
}

.solara-autorouter-content {
    max-width: 100%;
}

.jp-RenderedHTMLCommon {
    padding-right: 0;
}

/* MAILCHIMP */

#mc_embed_signup {
    background: transparent;
    clear: left;
    width: 100%;
}
#mc_embed_signup .indicates-required{
    margin-right: 0;
}
#mc_embed_signup .mc-field-group{
    width: 100%;
}
#mc_embed_signup_scroll > div.mc-field-group:nth-child(3){
    display: none;
}


/* MOBILE */

@media screen and (aspect-ratio < 1) {
    .hero{
        background-position-x: 29%;
    }
        .hero h2{
            padding-left: 0;
            padding-right: 0;
        }
    .container.fill-height > .ma-8.row{
        max-width: 80%;
    }
    .footer-wrapper{
        padding: 25px;
        row-gap: 1em;
    }
    .row-container{
        width: calc(100% - 16px);
        padding: 12px;
    }
        .row-container > div{
            max-width: 300px;
        }
}
