/* generated using: pygmentize -S vs -f html -S one-dark -a ".theme--dark.v-application .highlight" >! solara/server/static/highlight-dark.css
   and then remove the overlapping first lines with highlight.css
*/
.theme--dark.v-application .highlight .hll { background-color: #ffffcc }
.theme--dark.v-application .highlight { background: #282C34; color: #ABB2BF }
.theme--dark.v-application .highlight .c { color: #7F848E } /* Comment */
.theme--dark.v-application .highlight .err { color: #ABB2BF } /* Error */
.theme--dark.v-application .highlight .esc { color: #ABB2BF } /* Escape */
.theme--dark.v-application .highlight .g { color: #ABB2BF } /* Generic */
.theme--dark.v-application .highlight .k { color: #C678DD } /* Keyword */
.theme--dark.v-application .highlight .l { color: #ABB2BF } /* Literal */
.theme--dark.v-application .highlight .n { color: #E06C75 } /* Name */
.theme--dark.v-application .highlight .o { color: #56B6C2 } /* Operator */
.theme--dark.v-application .highlight .x { color: #ABB2BF } /* Other */
.theme--dark.v-application .highlight .p { color: #ABB2BF } /* Punctuation */
.theme--dark.v-application .highlight .ch { color: #7F848E } /* Comment.Hashbang */
.theme--dark.v-application .highlight .cm { color: #7F848E } /* Comment.Multiline */
.theme--dark.v-application .highlight .cp { color: #7F848E } /* Comment.Preproc */
.theme--dark.v-application .highlight .cpf { color: #7F848E } /* Comment.PreprocFile */
.theme--dark.v-application .highlight .c1 { color: #7F848E } /* Comment.Single */
.theme--dark.v-application .highlight .cs { color: #7F848E } /* Comment.Special */
.theme--dark.v-application .highlight .gd { color: #ABB2BF } /* Generic.Deleted */
.theme--dark.v-application .highlight .ge { color: #ABB2BF } /* Generic.Emph */
.theme--dark.v-application .highlight .gr { color: #ABB2BF } /* Generic.Error */
.theme--dark.v-application .highlight .gh { color: #ABB2BF } /* Generic.Heading */
.theme--dark.v-application .highlight .gi { color: #ABB2BF } /* Generic.Inserted */
.theme--dark.v-application .highlight .go { color: #ABB2BF } /* Generic.Output */
.theme--dark.v-application .highlight .gp { color: #ABB2BF } /* Generic.Prompt */
.theme--dark.v-application .highlight .gs { color: #ABB2BF } /* Generic.Strong */
.theme--dark.v-application .highlight .gu { color: #ABB2BF } /* Generic.Subheading */
.theme--dark.v-application .highlight .gt { color: #ABB2BF } /* Generic.Traceback */
.theme--dark.v-application .highlight .kc { color: #E5C07B } /* Keyword.Constant */
.theme--dark.v-application .highlight .kd { color: #C678DD } /* Keyword.Declaration */
.theme--dark.v-application .highlight .kn { color: #C678DD } /* Keyword.Namespace */
.theme--dark.v-application .highlight .kp { color: #C678DD } /* Keyword.Pseudo */
.theme--dark.v-application .highlight .kr { color: #C678DD } /* Keyword.Reserved */
.theme--dark.v-application .highlight .kt { color: #E5C07B } /* Keyword.Type */
.theme--dark.v-application .highlight .ld { color: #ABB2BF } /* Literal.Date */
.theme--dark.v-application .highlight .m { color: #D19A66 } /* Literal.Number */
.theme--dark.v-application .highlight .s { color: #98C379 } /* Literal.String */
.theme--dark.v-application .highlight .na { color: #E06C75 } /* Name.Attribute */
.theme--dark.v-application .highlight .nb { color: #E5C07B } /* Name.Builtin */
.theme--dark.v-application .highlight .nc { color: #E5C07B } /* Name.Class */
.theme--dark.v-application .highlight .no { color: #E06C75 } /* Name.Constant */
.theme--dark.v-application .highlight .nd { color: #61AFEF } /* Name.Decorator */
.theme--dark.v-application .highlight .ni { color: #E06C75 } /* Name.Entity */
.theme--dark.v-application .highlight .ne { color: #E06C75 } /* Name.Exception */
.theme--dark.v-application .highlight .nf { color: #61AFEF; font-weight: bold } /* Name.Function */
.theme--dark.v-application .highlight .nl { color: #E06C75 } /* Name.Label */
.theme--dark.v-application .highlight .nn { color: #E06C75 } /* Name.Namespace */
.theme--dark.v-application .highlight .nx { color: #E06C75 } /* Name.Other */
.theme--dark.v-application .highlight .py { color: #E06C75 } /* Name.Property */
.theme--dark.v-application .highlight .nt { color: #E06C75 } /* Name.Tag */
.theme--dark.v-application .highlight .nv { color: #E06C75 } /* Name.Variable */
.theme--dark.v-application .highlight .ow { color: #56B6C2 } /* Operator.Word */
.theme--dark.v-application .highlight .p-Marker { color: #ABB2BF } /* Punctuation.Marker */
.theme--dark.v-application .highlight .w { color: #ABB2BF } /* Text.Whitespace */
.theme--dark.v-application .highlight .mb { color: #D19A66 } /* Literal.Number.Bin */
.theme--dark.v-application .highlight .mf { color: #D19A66 } /* Literal.Number.Float */
.theme--dark.v-application .highlight .mh { color: #D19A66 } /* Literal.Number.Hex */
.theme--dark.v-application .highlight .mi { color: #D19A66 } /* Literal.Number.Integer */
.theme--dark.v-application .highlight .mo { color: #D19A66 } /* Literal.Number.Oct */
.theme--dark.v-application .highlight .sa { color: #98C379 } /* Literal.String.Affix */
.theme--dark.v-application .highlight .sb { color: #98C379 } /* Literal.String.Backtick */
.theme--dark.v-application .highlight .sc { color: #98C379 } /* Literal.String.Char */
.theme--dark.v-application .highlight .dl { color: #98C379 } /* Literal.String.Delimiter */
.theme--dark.v-application .highlight .sd { color: #98C379 } /* Literal.String.Doc */
.theme--dark.v-application .highlight .s2 { color: #98C379 } /* Literal.String.Double */
.theme--dark.v-application .highlight .se { color: #98C379 } /* Literal.String.Escape */
.theme--dark.v-application .highlight .sh { color: #98C379 } /* Literal.String.Heredoc */
.theme--dark.v-application .highlight .si { color: #98C379 } /* Literal.String.Interpol */
.theme--dark.v-application .highlight .sx { color: #98C379 } /* Literal.String.Other */
.theme--dark.v-application .highlight .sr { color: #98C379 } /* Literal.String.Regex */
.theme--dark.v-application .highlight .s1 { color: #98C379 } /* Literal.String.Single */
.theme--dark.v-application .highlight .ss { color: #98C379 } /* Literal.String.Symbol */
.theme--dark.v-application .highlight .bp { color: #E5C07B } /* Name.Builtin.Pseudo */
.theme--dark.v-application .highlight .fm { color: #56B6C2; font-weight: bold } /* Name.Function.Magic */
.theme--dark.v-application .highlight .vc { color: #E06C75 } /* Name.Variable.Class */
.theme--dark.v-application .highlight .vg { color: #E06C75 } /* Name.Variable.Global */
.theme--dark.v-application .highlight .vi { color: #E06C75 } /* Name.Variable.Instance */
.theme--dark.v-application .highlight .vm { color: #E06C75 } /* Name.Variable.Magic */
.theme--dark.v-application .highlight .il { color: #D19A66 } /* Literal.Number.Integer.Long */
