html,
body {
    height: 100vh;
}


@keyframes spin {
    0% {
        transform: rotate(0deg);
    }

    100% {
        transform: rotate(359deg);
    }
}

@keyframes oscillate {
    0% {
        transform: scale(0.8);
    }

    50% {
        transform: scale(1.0);
    }

    100% {
        transform: scale(0.8);
    }
}

#sun-spinner1 {
    animation: spin 6s linear infinite;
    padding: 10px;
}

#sun-spinner2 {
    animation: spin 2s cubic-bezier(.79, .14, .15, .86) infinite;
    padding: 10px;
}

#sun-spinner-container {
    animation: oscillate 2s linear infinite;
}

@keyframes bounce-in {
    0% {
        transform: scale(0);
    }

    100% {
        transform: scale(1);
    }
}

@keyframes bounce-out {
    0% {
        transform: translateZ(0) scale(1);
    }

    100% {
        transform: translateZ(0) scale(50);
    }
}

.solara-transition-enter-active {
    transition: opacity 0.5s ease;
    /* animation: bounce-in 0.5s ease; */
}

.solara-transition-leave-active {
    transition: opacity 0.5s ease;
}

.solara-transition-leave-active .solara-loader {
    animation: bounce-out 0.5s ease;
}

.solara-transition-leave-active #sun-spinner-container {
    animation: unset;
}

.solara-transition-leave-active h1 {
    visibility: hidden;
}

.solara-transition-enter,
.solara-transition-leave-to {
    opacity: 0;
}

#sun-spinner-container {
    overflow: hidden;
}

#loader-container {
    width: 100vw;
    height: 100vh;
    position: absolute;
    margin: 0;
    padding: 0;
    z-index: 998;
    display: flex;
    align-items: center;
}

#loader {
    position: absolute;
    text-align: center;
    z-index: 1000;
    overflow: hidden;
}
